@@ -10,12 +10,12 @@ PKG = os.path.dirname(os.path.dirname(__file__))
 def test(extra_args=None):
     try:
         import pytest
-    except ImportError:
-        raise ImportError("Need pytest>=5.0.1 to run tests")
+    except ImportError as err:
+        raise ImportError("Need pytest>=5.0.1 to run tests") from err
     try:
         import hypothesis  # noqa
-    except ImportError:
-        raise ImportError("Need hypothesis>=3.58 to run tests")
+    except ImportError as err:
+        raise ImportError("Need hypothesis>=3.58 to run tests") from err
     cmd = ["--skip-slow", "--skip-network", "--skip-db"]
     if extra_args:
         if not isinstance(extra_args, list):
